==> dovecot-0.9.tar.gz: dovecot-0.9/src/login/master.c <==
/*
    Copyright (C) 2002 Timo Sirainen

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "common.h"
#include "network.h"
#include "fdpass.h"
#include "master.h"

typedef struct _WaitingRequest WaitingRequest;

struct _WaitingRequest {
	WaitingRequest *next;

	int id;
	MasterCallback callback;
	void *user_data;
};

static IO io_master;
static WaitingRequest *requests, **next_request;

static int master_pos;
static char master_buf[sizeof(MasterReply)];

static void push_request(int id, MasterCallback callback, void *user_data)
{
	WaitingRequest *req;

	req = i_new(WaitingRequest, 1);
	req->id = id;
	req->callback = callback;
	req->user_data = user_data;

	*next_request = req;
	next_request = &req->next;
}

static void pop_request(MasterReply *reply)
{
	WaitingRequest *req;

	req = requests;
	if (req == NULL) {
		i_warning("Master sent us unrequested reply for id %d",
			  reply->id);
		return;
	}

	if (reply->id != req->id) {
		i_fatal("Master sent invalid id for reply "
			"(got %d, expecting %d)", reply->id, req->id);
	}

	req->callback(reply->success, req->user_data);

	requests = req->next;
	if (requests == NULL)
		next_request = &requests;

	i_free(req);
}

void master_send_login(const char *user, const char *pass, int fd,
		       const char *login_tag, MasterCallback callback,
		       void *user_data)
{
	MasterRequest req;

	i_assert(strlen(user) < sizeof(req.user));
	i_assert(strlen(pass) < sizeof(req.pass));
	i_assert(fd > 1);

	memset(&req, 0, sizeof(req));
	req.id = fd;
	strcpy(req.user, user);
	strcpy(req.pass, pass);

	if (strlen(login_tag) >= sizeof(req.login_tag))
		strcpy(req.login_tag, "*");
	else
		strcpy(req.login_tag, login_tag);

	if (fd_send(MASTER_SOCKET_FD, fd, &req, sizeof(req)) != sizeof(req))
		i_fatal("fd_send() failed: %s", strerror(errno));

	push_request(fd, callback, user_data);
}

void master_send_uid(uid_t uid, gid_t gid, const char *user,
		     const char *chroot_dir, const char *mail, int fd,
		     const char *login_tag, MasterCallback callback,
		     void *user_data)
{
	MasterRequest req;

	if (user == NULL) user = "";
	if (chroot_dir == NULL) chroot_dir = "";
	if (mail == NULL) mail = "";

	i_assert(strlen(user) < sizeof(req.user));
	i_assert(strlen(chroot_dir) < sizeof(req.chroot_dir));
	i_assert(strlen(mail) < sizeof(req.mail));
	i_assert(fd > 1);

	memset(&req, 0, sizeof(req));
	req.id = fd;
	req.uid = uid;
	req.gid = gid;
	strcpy(req.user, user);
	strcpy(req.chroot_dir, chroot_dir);
	strcpy(req.mail, mail);

	if (strlen(login_tag) >= sizeof(req.login_tag))
		strcpy(req.login_tag, "*");
	else
		strcpy(req.login_tag, login_tag);

	memset(&req, 0, sizeof(req));
	if (fd_send(MASTER_SOCKET_FD, fd, &req, sizeof(req)) != sizeof(req))
		i_fatal("fd_send() failed: %s", strerror(errno));

	push_request(fd, callback, user_data);
}

static void master_input(void *user_data, int fd, IO io)
{
	int ret;

	ret = net_receive(fd, master_buf + master_pos,
			  sizeof(master_buf) - master_pos);
	if (ret < 0) {
		/* master died, kill ourself too */
		io_loop_stop(ioloop);
	}

	master_pos += ret;
	if (master_pos < sizeof(master_buf))
		return;

	/* reply is now read */
	pop_request((MasterReply *) master_buf);
	master_pos = 0;
}

void master_init(void)
{
	requests = NULL;
	next_request = &requests;

        master_pos = 0;
	io_master = io_add(MASTER_SOCKET_FD, IO_READ, master_input, NULL);
}

void master_deinit(void)
{
	WaitingRequest *next;

	while (requests != NULL) {
		next = requests->next;
		i_free(requests);
		requests = next;
	}
	io_remove(io_master);
}

==> dovecot-0.92.tar.gz: dovecot-0.92/src/login/master.c <==
/*
    Copyright (C) 2002 Timo Sirainen

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "common.h"
#include "network.h"
#include "fdpass.h"
#include "master.h"

typedef struct _WaitingRequest WaitingRequest;

struct _WaitingRequest {
	WaitingRequest *next;

	int id;
	MasterCallback callback;
	void *user_data;
};

static IO io_master;
static WaitingRequest *requests, **next_request;

static int master_pos;
static char master_buf[sizeof(MasterReply)];

static void push_request(int id, MasterCallback callback, void *user_data)
{
	WaitingRequest *req;

	req = i_new(WaitingRequest, 1);
	req->id = id;
	req->callback = callback;
	req->user_data = user_data;

	*next_request = req;
	next_request = &req->next;
}

static void pop_request(MasterReply *reply)
{
	WaitingRequest *req;

	req = requests;
	if (req == NULL) {
		i_error("Master sent us unrequested reply for id %d",
			reply->id);
		return;
	}

	if (reply->id != req->id) {
		i_fatal("Master sent invalid id for reply "
			"(got %d, expecting %d)", reply->id, req->id);
	}

	req->callback(reply->success, req->user_data);

	requests = req->next;
	if (requests == NULL)
		next_request = &requests;

	i_free(req);
}

void master_request_imap(int fd, int auth_process, const char *login_tag,
			 unsigned char cookie[AUTH_COOKIE_SIZE],
			 MasterCallback callback, void *user_data)
{
	MasterRequest req;

	i_assert(fd > 1);

	memset(&req, 0, sizeof(req));
	req.id = fd;
	req.auth_process = auth_process;
	memcpy(req.cookie, cookie, AUTH_COOKIE_SIZE);

	if (strlen(login_tag) >= sizeof(req.login_tag))
		strcpy(req.login_tag, "*");
	else
		strcpy(req.login_tag, login_tag);

	if (fd_send(MASTER_SOCKET_FD, fd, &req, sizeof(req)) != sizeof(req))
		i_fatal("fd_send() failed: %s", strerror(errno));

	push_request(req.id, callback, user_data);
}

static void master_input(void *user_data, int fd, IO io)
{
	int ret;

	ret = net_receive(fd, master_buf + master_pos,
			  sizeof(master_buf) - master_pos);
	if (ret < 0) {
		/* master died, kill ourself too */
		io_loop_stop(ioloop);
		return;
	}

	master_pos += ret;
	if (master_pos < sizeof(master_buf))
		return;

	/* reply is now read */
	pop_request((MasterReply *) master_buf);
	master_pos = 0;
}

void master_init(void)
{
	requests = NULL;
	next_request = &requests;

        master_pos = 0;
	io_master = io_add(MASTER_SOCKET_FD, IO_READ, master_input, NULL);
}

void master_deinit(void)
{
	WaitingRequest *next;

	while (requests != NULL) {
		next = requests->next;
		i_free(requests);
		requests = next;
	}
	io_remove(io_master);
}
